/*
 * Copyright 2009-2010 Nanjing RedOrange ltd (http://www.red-orange.cn)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package redora.client.mvp;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import redora.client.Persistable;
import redora.client.validation.BusinessRuleViolation;

/**
 * After a persist is invoked in the service object, the service
 * object will fire an PersistEvent when it has retrieved a response
 * from the server. The result can either be 'success': the event
 * will then carry the persisted pojo, or 'violation': the package
 * then is a list of {@link BusinessRuleViolation}s.
 * @see ServiceBase#persist(redora.client.Persistable)
 * @author Nanjing RedOrange (www.red-orange.cn)
 */
public class PersistEvent extends GwtEvent<PersistEvent.Handler> {
    public static Type<PersistEvent.Handler> TYPE = new Type<PersistEvent.Handler>();

    /**
 * Note that there is no onError event. When an error occurs, this
 * will be notified directly to the info/error message label.
 * @author Nanjing RedOrange (www.red-orange.cn)
 */
public interface Handler extends EventHandler {
    /** Persisted successfully. The payload is a persisted pojo.*/
    void onSuccess(PersistEvent event);
    /** Business rules were violated. See event.violations for the violations.*/
    void onViolation(PersistEvent event);
}

    public final Persistable pojo;
    public final BusinessRuleViolation[] violations;
    public final Class<? extends Persistable> cls;
    public boolean wasNew;

    public PersistEvent(Class<? extends Persistable> cls, Persistable pojo, boolean wasNew) {
        this.cls = cls;
        this.pojo = pojo;
        this.wasNew = wasNew;
        violations = null;
    }

    public PersistEvent(Class<? extends Persistable> cls, Persistable pojo, BusinessRuleViolation[] violations, boolean wasNew) {
        this.cls = cls;
        this.pojo = pojo;
        pojo = null;
        this.violations = violations;
        this.wasNew = wasNew;
    }

    @Override
    public Type<PersistEvent.Handler> getAssociatedType() {
        return TYPE;
    }

    @Override
    protected void dispatch(PersistEvent.Handler handler) {
        if (violations == null)
            handler.onSuccess(this);
        else
            handler.onViolation(this);
    }
}
